# Połączenie ze zdalnym środowiskiem
- Uruchom program noMachine i połącz się ze zdalnym komputerem przydzielonym przez prowadzącego

Przypomnienie [lab00](https://git.pg.edu.pl/p962302/ppb_tpa_lab2020/tree/master/lab00)
 
# Instalacja oraz uruchomienie punktu dostępowego

## Architektura sieci -  poglądowych schemat
![Network architecture](img/network_sch.png "Network architecture")

## Instalacja i konfiguracja Access Point (AP)
![AP configure](img/ap_configure.gif "Access point configure")
https://roboticsknowledgebase.com/wiki/networking/wifi-hotspot/

## Przydatne komendy do diagnozowania problemów z siecią
```bash
lspci
lsusb
iw list
iwconfig
route -n
ls /dev/serial/by-id -al
```
Diagnostyka połączenia:
```bash
sudo tcpdump -i wlp0s20f3 icmp
sudo tcpdump -i eno1 icmp
```

# Oprogramowanie ESP32
## Instalacja biblioteki umożliwiającej programowanie układów ESP32 z poziomu Arduino IDE
https://docs.espressif.com/projects/arduino-esp32/en/latest/installing.html
1. Preference -> Additional Board Manager URLs -> dodać link "https://raw.githubusercontent.com/espressif/arduino-esp32/gh-pages/package_esp32_index.json"
2. Tools->Board -> zainstaluj esp32
3. Restart IDE

![Add ESP32 in Arduino ID](img/esp32_arduino.png "Add ESP32 in Arduino IDE")

## Ćwiczenie 1 - wstępne:
Celem pierwszego ćwiczenia jest skonfigurowanie połączenia bezprzewodowego z układem esp32.
 - uruchomienie AP
 - skanowanie dostępnych AP z poziomu ESP32
 - połączenie esp32 z AP (statyczny adres IP) - ping lokalny AP->ESP32
 - instalacja biblioteki ESP32Ping  - ping lokalny ESP32->AP
 - konfiguracja zapory na AP - ping zewnętrzny ESP32->8.8.8.8
 - statyczny serwer DNS - ping zewnętrzny ESP32->google.com
 - serwer DHCP - dynamiczna konfiguracja
 
**Wskazówki:**
- resetowanie Wifi - potrzebne do wejścia w tryb scan
```cpp
  #include <WiFi.h>
  void resetWiFi()
{
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(3000); //After some delay to let the module disconnect, go to scanning for available networks
                       // wait for a second
}
```
- skanowanie dostępnych AP
```cpp
#include <WiFi.h>
void scanSSID()
{
  Serial.print("Scan start ... ");
  int n = WiFi.scanNetworks();
  Serial.print(n);
  Serial.println(" network(s) found");
  for (int i = 0; i < n; i++)
  {
    Serial.println(WiFi.SSID(i));
  }
  Serial.println();
}
```
- statyczna/dynamiczna konfiguracja sieci
```cpp
#include <WiFi.h>
void connectSSID(const char * ssid, const char* password)
{
  WiFi.begin(ssid, password);
// statyczna
//  IPAddress ip(192,168,0,200);   
//  IPAddress gateway(192,168,0,1);   
//  IPAddress subnet(255,255,255,0);
//  IPAddress dns(8,8,8,8); 
//  WiFi.config(ip, gateway, subnet, dns, dns);
// domyslnie dynamiczna
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected with AP");
  Serial.println("");
}
```
- wyświetlenie parametrów połączenia bezprzewodowego
```cpp
#include <WiFi.h>
void printConnectionParam()
{
  Serial.println("WiFi connected!");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("ESP Mac Address: ");
  Serial.println(WiFi.macAddress());
  Serial.print("Subnet Mask: ");
  Serial.println(WiFi.subnetMask());
  Serial.print("Gateway IP: ");
  Serial.println(WiFi.gatewayIP());
  Serial.print("DNS: ");
  Serial.println(WiFi.dnsIP());  
}
```
- sprawdzenia połączenia za pomocą usługi ICMP/Ping
Instalacja dodatkowej biblioteki ESP32Ping
Arduino IDE -> Sketch -> Include Library -> Add .ZIP Lib, dodać link do biblioteki
https://github.com/marian-craciunescu/ESP32Ping
![Add ZIP lib in Arduino IDE](img/esp32_addZipLib.png "Add ZIP lib in Arduino IDE")

```cpp
#include <ESP32Ping.h>
```
```cpp
void pingIP(unsigned int a1, unsigned int a2 , unsigned int a3 , unsigned int a4)
{
  IPAddress ip(a1,a2,a3,a4);
  if(Ping.ping(ip)) 
  {
    Serial.printf("Ping IP=%u.%u.%u.%u Success!!\n", a1, a2, a3, a4);
  }
  else
  {
    Serial.printf("Ping IP=%u.%u.%u.%u Error!!\n" , a1, a2, a3, a4);
  }
}

void pingDomain(const char* domain)
{
  if(Ping.ping(domain)) 
  {
    Serial.printf("Ping domain=%s Success!!\n", domain);
  }
  else
  {
    Serial.printf("Ping domain=%s Error!!\n", domain);
  }
}
```
**Zadanie 1:** Celem pierwszego ćwiczenia jest pobranie w esp32 aktualnej daty i czasu z z serwera NTP oraz wyświetlenie pobranej daty na konsoli. Następnie w sposób ciągły (w funkcji loop) należy sterować diodą LED (pin 12), w następujący sposób:
 - LED=HIGH: gdy dziesiętna liczba sekund aktualnego czasu jest parzysta;
 - LED=LOW: gdy dziesiętna liczba sekund aktualnego czasu jest nieparzysta;
 - LED=BLINK(5razy@1s): gdy liczba sekund aktualnego czasu równa sie zero.
   
 **Wyjaśnienia:**
Sekwencja sterowania LED powinna wyglądać następująco: przez 10sek LED świeci, przez kolejne 10s nie świeci, ten schemat powtarza się trzykrotnie wówczas następuje 1s szybkie mruganie.  

 - **wskazówki:** 
 - W pliku nagłówkowym #include "time.h" znajduje sie deklaracja funkcji 
  void configTime(int timezone_sec, int daylightOffset_sec, const char* server1, const char* server2, const char* server3)
  - Datę z serwera NTP można pobrać raz np. co minutę, wbudowany timer w esp będzie sie automatycznie aktualizował.
```cpp
#include "time.h"
//void configTime(int timezone_sec, int daylightOffset_sec, const char* server1, const char* server2, const char* server3)
void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}
```
