#include <WiFi.h>
#include <ESP32Ping.h>
#include "time.h"

#define LED_BUILTIN 12

void initSerial()
{
  Serial.begin(9600);
  while (!Serial){  }
  Serial.println();
  Serial.println("...Start..");  //serial monitor gubi kilka pierwszych znakow/lini
  Serial.printf("Compile time: %s %s\n", __DATE__, __TIME__);
}

void scanSSID()
{
  Serial.print("Scan start ... ");
  int n = WiFi.scanNetworks();
  Serial.print(n);
  Serial.println(" network(s) found");
  for (int i = 0; i < n; i++)
  {
    Serial.println(WiFi.SSID(i));
  }
  Serial.println();
}

void resetWiFi()
{
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(3000); //After some delay to let the module disconnect, go to scanning for available networks
                       // wait for a second
}

void connectSSID(const char * ssid, const char* password)
{
  WiFi.begin(ssid, password);
// statyczna
//  IPAddress ip(192,168,0,200);   
//  IPAddress gateway(192,168,0,1);   
//  IPAddress subnet(255,255,255,0);
//  IPAddress dns(8,8,8,8); 
//  WiFi.config(ip, gateway, subnet, dns, dns);
// domyslnie dynamiczna
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected with AP");
  Serial.println("");
}


void printConnectionParam()
{

  Serial.println("WiFi connected!");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("ESP Mac Address: ");
  Serial.println(WiFi.macAddress());
  Serial.print("Subnet Mask: ");
  Serial.println(WiFi.subnetMask());
  Serial.print("Gateway IP: ");
  Serial.println(WiFi.gatewayIP());
  Serial.print("DNS: ");
  Serial.println(WiFi.dnsIP());  
}


void pingIP(unsigned int a1, unsigned int a2 , unsigned int a3 , unsigned int a4)
{
  IPAddress ip(a1,a2,a3,a4);
  if(Ping.ping(ip)) 
  {
    Serial.printf("Ping IP=%u.%u.%u.%u Success!!\n", a1, a2, a3, a4);
  }
  else
  {
    Serial.printf("Ping IP=%u.%u.%u.%u Error!!\n" , a1, a2, a3, a4);
  }
}

void pingDomain(const char* domain)
{
  if(Ping.ping(domain)) 
  {
    Serial.printf("Ping domain=%s Success!!\n", domain);
  }
  else
  {
    Serial.printf("Ping domain=%s Error!!\n", domain);
  }
}

void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}



// the setup function runs once when you press reset or power the board
void setup() {
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_BUILTIN, OUTPUT);
  initSerial();
  resetWiFi();
  scanSSID();
  connectSSID("TPA7", "tpa12345");
  printConnectionParam();
  pingIP(10,42,0,1);
  pingIP(8, 8,8,8);
  pingDomain("google.com");
  pingDomain("0.pl.pool.ntp.org");
  configTime(3600, 3600, "0.pl.pool.ntp.org");
  printLocalTime();
}

// the loop function runs over and over again forever
void loop() {
  digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
  delay(500);                       // wait for a second
  digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
  delay(500);                       // wait for a second
}